; =============================================================================
;	MSX definition header file
; -----------------------------------------------------------------------------
;	2019/07/03	t.hara
; =============================================================================

; =============================================================================
;	I/O MAP
; =============================================================================
IO_VDP_PORT0_MSX1_ADP		:= 0x88
IO_VDP_PORT1_MSX1_ADP		:= 0x89
IO_VDP_PORT2_MSX1_ADP		:= 0x8A
IO_VDP_PORT3_MSX1_ADP		:= 0x8B
IO_VDP_PORT0				:= 0x98
IO_VDP_PORT1				:= 0x99
IO_VDP_PORT2				:= 0x9A
IO_VDP_PORT3				:= 0x9B

IO_PSG_ADR					:= 0xA0
IO_PSG_DATA_WR				:= 0xA1
IO_PSG_DATA_RD				:= 0xA2

IO_BASE_SLOT				:= 0xA8

IO_MEMMAP0					:= 0xFC
IO_MEMMAP1					:= 0xFD
IO_MEMMAP2					:= 0xFE
IO_MEMMAP3					:= 0xFF

; =============================================================================
;	BIOS (MAIN-ROM)
; =============================================================================

;	for BASIC
CHKRAM						:= 0x0000
SYNCHR						:= 0x0008
RDSLT						:= 0x000C
CHRGTR						:= 0x0010
WRSLT						:= 0x0014
OUTDO						:= 0x0018
CALSLT						:= 0x001C
DCOMPR						:= 0x0020
ENASLT						:= 0x0024
GETYPR						:= 0x0028
CALLF						:= 0x0030
KEYINT						:= 0x0038
INITIO						:= 0x003B
INIFNK						:= 0x003E

;	for VDP access
DISSCR						:= 0x0041
ENASCR						:= 0x0047
RDVRM						:= 0x004A
WRVRM						:= 0x004D
SETRD						:= 0x0050
SETWRT						:= 0x0053
FILVRM						:= 0x0056
LDIRMV						:= 0x0059
LDIRVM						:= 0x005C
CHGMOD						:= 0x005F
CHGCLR						:= 0x0062
NMI							:= 0x0066
CLRSPR						:= 0x0069
INITTXT						:= 0x006C
INIT32						:= 0x006F
INIGRP						:= 0x0072
INIMLT						:= 0x0075
SETTXT						:= 0x0078
SETT32						:= 0x007B
SETGRP						:= 0x007E
SETMLT						:= 0x0081
CALPAT						:= 0x0084
CALATR						:= 0x0087
GSPSIZ						:= 0x008A
GRPPRT						:= 0x008D

;	for PSG
GICINI						:= 0x0090
WRTPSG						:= 0x0093
RDPSG						:= 0x0096
STRTMS						:= 0x0099

;	for Keyboard, CRT, Printer


; =============================================================================
;	SYSTEM WORK AREA
; =============================================================================

;	SUB-ROM SLOT
;	bit   7    6    5    4    3    2    1    0
;	    [ext][N/A][N/A][N/A][EXT.SLOT][BASESLOT]
;
EXBRSA						:= 0xFAF8

;	BASE SLOT EXTEND INFORMATION
;
EXPTBL0						:= 0xFCC1		; [ext][N/A][N/A][N/A][EXT.SLOT][BASESLOT]	MAIN-ROM slot
EXPTBL1						:= 0xFCC2		; [ext][N/A][N/A][N/A][N/A][N/A][N/A][N/A]	slot#1
EXPTBL2						:= 0xFCC3		; [ext][N/A][N/A][N/A][N/A][N/A][N/A][N/A]	slot#2
EXPTBL3						:= 0xFCC4		; [ext][N/A][N/A][N/A][N/A][N/A][N/A][N/A]	slot#3

;	VSYNC INTERRUPT HOOK
;	59.94Hz in NTSC, 50Hz in PAL
;
H_TIMI						:= 0xFD9F

;	USER AREA TOP LIMIT ADDRESS
;
HIMEM						:= 0xFC4A

; =============================================================================
;	DEFAULT VRAM MAP
; =============================================================================

;	screen 0 (width40)
SC0_W40_PAT_NAME			:= 0x0000
SC0_W40_PAT_GEN				:= 0x0800

;	screen 0 (width80)
SC0_W80_PAT_NAME			:= 0x0000
SC0_W80_BLINK				:= 0x0800
SC0_W80_PAT_GEN				:= 0x1000

;	screen 1
SC1_PAT_GEN					:= 0x0000
SC1_PAT_NAME				:= 0x1800
SC1_SPR_ATTR				:= 0x1B00
SC1_SPR_GEN					:= 0x3800

;	screen 2
SC2_PAT_GEN0				:= 0x0000
SC2_PAT_GEN1				:= 0x0800
SC2_PAT_GEN2				:= 0x1000
SC2_PAT_NAME				:= 0x1800
SC2_SPR_ATTR				:= 0x1B00
SC2_PAT_COL0				:= 0x2000
SC2_PAT_COL1				:= 0x2800
SC2_PAT_COL2				:= 0x3000
SC2_SPR_GEN					:= 0x3800

;	screen 3
SC3_PAT_GEN					:= 0x0000
SC3_PAT_NAME				:= 0x0800
SC3_SPR_ATTR				:= 0x1B00
SC3_SPR_GEN					:= 0x3800

;	screen 4
SC4_PAT_GEN0				:= 0x0000
SC4_PAT_GEN1				:= 0x0800
SC4_PAT_GEN2				:= 0x1000
SC4_PAT_NAME				:= 0x1800
SC4_SPR_COL					:= 0x1C00
SC4_SPR_ATTR				:= 0x1E00
SC4_PAT_COL0				:= 0x2000
SC4_PAT_COL1				:= 0x2800
SC4_PAT_COL2				:= 0x3000
SC4_SPR_GEN					:= 0x3800

; =============================================================================
;	MACROs
; =============================================================================
ROM_HEADER					MACRO	init
								defs	"AB"			; ID
								defw	init			; INIT
								defw	0				; STATEMENT
								defw	0				; DEVICE
								defw	0				; TEXT
								defb	0,0,0,0,0,0		; RESERVE
							ENDM

BSAVE_HEADER				MACRO	saddr, eaddr, exec
								defb	0xFE
								defw	saddr
								defw	eaddr
								defw	exec
							ENDM

ROM_ALIGN					MACRO
								if FILE_ADDRESS % 16384
									defs	" " * (16384 - (FILE_ADDRESS % 16384))
								endif
							ENDM
