// --------------------------------------------------------------------
//	ZMA: CZMA_TEXT
// ====================================================================
//	2019/05/02	t.hara
// --------------------------------------------------------------------

#include "zma_text.hpp"
#include <fstream>
#include <iostream>

// ----------------------------------------------------------------
//	Constructor
// ----------------------------------------------------------------
CZMA_TEXT::CZMA_TEXT() {
	code_size = 0;
	next_code_address = 0;
}

// ----------------------------------------------------------------
//	Destructor
// ----------------------------------------------------------------
CZMA_TEXT::~CZMA_TEXT() {
	for( auto p: m_text ) { 
		delete p; 
	}
}

// ----------------------------------------------------------------
//	File Load
// ----------------------------------------------------------------
bool CZMA_TEXT::load( CZMA_INFORMATION &info, const char* p_file_name ) {
	m_text.clear();
	char s_buffer[2048];
	std::string s;
	CZMA_PARSE *p_parse;
	int line_no;

	std::ifstream file;
	file.open( p_file_name );
	if( !file ) {
		return false;
	}
	line_no = 1;
	for( ;; line_no++) {
		s = "";
		for( ;;) {
			file.getline( s_buffer, sizeof( s_buffer ) );
			if( file.bad() || file.eof() ) {
				break;
			}
			s += s_buffer;
			if( file.fail() ) {
				file.clear( file.rdstate() & ~std::ios_base::failbit );
			}
			else {
				break;
			}
		}
		if( file.bad() || file.eof() ) {
			break;
		}
		p_parse = CZMA_PARSE::create( info, s, p_file_name, line_no );
		m_text.push_back( p_parse );
	}
	file.close();
	return true;
}

// --------------------------------------------------------------------
CZMA_PARSE *CZMA_TEXT::process( CZMA_INFORMATION &info, unsigned int &success_count, CZMA_PARSE *p_prev_line, bool output_mode ) {
	CZMA_PARSE* p_prev;
	bool result;
	int nest_count = 0;
	p_prev = p_prev_line;
	result = true;
	success_count = 0;
	for( auto p = m_text.begin(); p != m_text.end(); ) {
		if( output_mode ) {
			(*p)->set_output_mode();
		}
		if( info.is_block_processing ) {
			//	ubN̒̏
			if( (*p)->words.size() >= 1 && info.block_end_table.count( (*p)->words[0] ) ) {
				//	ubNL𔭌
				if( (*p)->words[0] == "ENDR" || (*p)->words[0] == "ENDM" || (*p)->words[0] == "ENDIF" ) {
					//	 ENDR, ENDM, ENDIF ̏
					if( nest_count ) {
						//	ubÑubN̂ŃlXg炷
						nest_count--;
						info.p_text->push_back( *p );
						p = m_text.erase( p );
						continue;
					}
				}
				if( info.block_type != info.block_end_table[(*p)->words[0]] ) {
					//	ڃubN̎ނƈقȂuLv𔭌
					(*p)->put_structure_error( "Invalid block pair." );
					p_prev = (*p);
					p++;
					continue;
				}
				//	uLv̏
				if( (*p)->process( info, p_prev ) ) {
					success_count++;
				}
				p_prev = (*p);
				p++;
			}
			else {
				if( (*p)->words.size() >= 1 && info.block_begin_table.count( (*p)->words[0] ) ) {
					//	ubN̓ɃubN̊Jn𔭌
					if( (*p)->words[0] == "REPEAT" || (*p)->words[0] == "IF" ) {
						nest_count++;
					}
				}
				else if( (*p)->words.size() >= 2 && (*p)->words[1] == "MACRO" ) {
					//	ubN̓Ƀ}N錾𔭌
					(*p)->put_structure_error( "MACRO cannot define in MACRO/REPEAT/IF block." );
					p_prev = (*p);
					p++;
					continue;
				}
				info.p_text->push_back( *p );
				p = m_text.erase( p );
			}
		}
		else {
			//	p[XG[Ns̏ꍇAăp[X݂
			if( (*p)->is_parse_error() ) {
				CZMA_PARSE *p_parse = CZMA_PARSE::create( info, (*p)->get_line(), (*p)->get_file_name(), (*p)->get_line_no() );
				if( !(p_parse->is_parse_error()) ) {
					p = m_text.erase( p );
					p = m_text.insert( p, p_parse );
				}
				else {
					delete p_parse;
				}
			}
			//	ubN̊Ȍ
			if( (*p)->process( info, p_prev ) ) {
				success_count++;
			}
			p_prev = (*p);
			p++;
		}
	}
	return p_prev;
}

// --------------------------------------------------------------------
bool CZMA_TEXT::all_process( CZMA_INFORMATION& info ) {
	unsigned int success_count, previous_success_count;
	CZMA_PARSE* p_last_line;

	previous_success_count = 0;
	for( ; ; ) {
		success_count = 0;
		info.scope.clear();
		info.is_block_processing = false;
		info.auto_label_index = 0;
		p_last_line = this->process( info, success_count, nullptr, false );
		if( info.is_block_processing ) {
			p_last_line->set_output_mode();
			p_last_line->put_error( "Block processing is not close." );
			break;
		}
		if( success_count == previous_success_count ) {
			break;
		}
		previous_success_count = success_count;
	}
	info.scope.clear();
	info.is_block_processing = false;
	info.auto_label_index = 0;
	p_last_line = this->process( info, success_count, nullptr, true );
	if( p_last_line == nullptr ) {
		std::cerr << "Code is not found.\n";
		return false;
	}
	if( info.scope.size() > 0 ) {
		p_last_line->put_error( "Scope is not closed." );
		return false;
	}
	if( CZMA_PARSE::get_number_of_errors() ) {
		std::cerr << "Found " << CZMA_PARSE::get_number_of_errors() << " error(s).\n";
		return false;
	}
	return true;
}

// --------------------------------------------------------------------
bool CZMA_TEXT::write( CZMA_INFORMATION& info, std::ofstream* f ) {
	bool result;

	result = true;
	for( auto p: m_text ) {
		result = result & p->write( info, f );
	}
	return result;
}

// --------------------------------------------------------------------
bool CZMA_TEXT::save( CZMA_INFORMATION& info, const char* p_file_name ) {
	std::ofstream f;

	f.open( p_file_name, std::ios::out | std::ios::binary );
	bool result = this->write( info, &f );
	info.write();
	return result;
}
